package org.xbib.interlibrary.action.sru;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import org.xbib.interlibrary.api.BibliographicDescription;
import org.xbib.interlibrary.api.action.Request;

/**
 * Contextual Query Language request for Elasticsearch
 */
public class SearchRetrieveRequest implements Request {

    private final Builder builder;

    private SearchRetrieveRequest(Builder builder) {
        this.builder = builder;
    }

    public static Builder builder() {
        return new Builder();
    }

    public String getContent() {
        return builder.content;
    }

    public Map<String, Object> getMap() {
        return builder.map;
    }

    public BibliographicDescription getBibliographicDescription() {
        return builder.bibliographicDescription;
    }

    public String getSearchString() {
        return builder.searchString;
    }

    public String getSearchKey() {
        return builder.searchKey;
    }

    public String getEndPoint() {
        return builder.endpoint;
    }

    public String getFilter() {
        return builder.filter;
    }

    @Override
    public Collection<Object> getServices() {
        return builder.services;
    }

    @Override
    public SearchRequester getRequester() {
        return builder.requester;
    }

    @Override
    public Integer getOffset() {
        return builder.startRecord != null ? builder.startRecord - 1 : 0;
    }

    @Override
    public Integer getSize() {
        return builder.maximumRecords != null ? builder.maximumRecords : 10;
    }

    public String getVersion() {
        return builder.version;
    }

    public String getRecordSchema() {
        return builder.recordSchema;
    }

    public String getExtraRequestData() {
        return builder.extraRequestData;
    }

    public Integer getStartRecord() {
        return builder.startRecord != null ? builder.startRecord : 0;
    }

    public Integer getMaximumRecords() {
        return builder.maximumRecords != null ? builder.maximumRecords : 10;
    }

    public String getFacetLimit() {
        return builder.facetLimit;
    }

    public String getFacetSort() {
        return builder.facetSort;
    }

    public String getStyleSheet() {
        return builder.stylesheet;
    }

    public String getDefaultStyleSheet() {
        return builder.defaultStylesheet;
    }

    public String getPath() {
        return builder.path;
    }

    public Map<String, Collection<String>> getOrFilter() {
        return builder.orFilter;
    }

    public Map<String, Collection<String>> getAndFilter() {
        return builder.andFilter;
    }

    public static class Builder {

        private SearchRequester requester;

        private String content;

        private Map<String, Object> map;

        private String searchString;

        private String searchKey;

        private BibliographicDescription bibliographicDescription;

        private Integer offset = 0;

        private Integer size = 10;

        private String endpoint;

        private String filter;

        private Collection<Object> services;

        private String version = "2.0";

        /**
         * In order that records which are not well formed do not break the entire message, it is possible to
         * request that they be transferred as a single string with the < > and & escaped to their entity forms.
         * Moreover some toolkits may not be able to distinguish record XML from the XML which forms the response.
         * However, some clients may prefer that the records be transferred as XML in order to manipulate them
         * directly with a stylesheet which renders the records and potentially also the user interface.
         * This distinction is made via the recordPacking parameter in the request. If the value of the
         * parameter is 'string', then the server should escape the record before transfering it.
         * If the value is 'xml', then it should embed the XML directly into the response.
         * Either way, the data is transfered within the 'recordData' field.
         * If the server cannot comply with this packing request, then it must return a diagnostic.
         */
        private String recordPacking;

        /**
         * The schema requested for the records to be returned.
         * If the recordXPath parameter (next) is included, it is the abstract schema for purposes of
         * evaluation by the XPath expression.
         * If the recordXPath parameter is not included, it is the schema that response records should assume.
         * The value is the URI identifer for the schema or the short name for it published by the server.
         * The default value if not supplied is determined by the server (see explain).
         */
        private String recordSchema;

        private String cqlQuery;

        private Map<String, Collection<String>> orFilter;

        private Map<String, Collection<String>> andFilter;

        /**
         * The position within the sequence of matched records (see result set model) of the first record to be
         * returned. The first position in the sequence is 1. The value supplied must be greater than 0.
         * Default value if not supplied is 1.
         */
        private Integer startRecord;

        /**
         * The number of records requested to be returned. The value must be 0 or greater. Default value
         * if not supplied is determined by the server (see explain). The server may return less than this
         * number of records, for example if there are fewer matching records than requested.
         */
        private Integer maximumRecords;

        /**
         * The number of seconds for which the client requests that the result set created should be maintained.
         * The server may choose not to fulfil this request, and may respond with a different number of seconds
         * (parameter resultSetIdleTime in response). If resultSetTTL is not supplied then the server will
         * determine the value.
         */
        private Integer resultSetTTL;

        /**
         * Sorting
         * A request may include a sort specification, indicating the desired ordering of the results.
         * This is a request for the server to apply a sorting algorithm to the result set before
         * returning any records. It may be supplied with a new search or applied to an existing result set.
         * The sort parameter is included in the searchRetrieve operation, rather than defined as a separate
         * operation, for two reasons: (1) if the server knows the desired sort order in advance,
         * query processing can be optimized (2) a server may be able to sort a result set at creation,
         * but not maintain it across multiple requests.
         * In order to specify result set(s) to sort, the query should include:
         * cql.resultSetId = "resultSetId"
         * where 'resultSetId' is the identifier supplied by the server for the result set.
         * If multiple result set identifiers are supplied, linked by boolean OR, AND or NOT, then
         * the request will combine and sort all of the given sets together. This is documented in the CQL context set.
         * The sort parameter includes one or more keys, each of which includes the following information:
         * path xsd:string Mandatory
         * An XPath expression describing a tagpath to be used in the sort. See additional description.
         * schema
         * xsd:string
         * Optional
         * The URI identifier for a supported schema. This schema is the one to which the XPath expression applies.
         * If it is not supplied then the default value from Explain will be used. See additional description.
         * ascending
         * xsd:boolean
         * Optional
         * Should the results be sorted ascending (true, and the default if not supplied) or descending (false).
         * caseSensitive
         * xsd:boolean
         * Optional
         * Should case be considered as important during the sort. The default value is false if not supplied.
         * missingValue
         * xsd:string
         * Optional
         * One of 'abort', 'highValue', 'lowValue', 'omit' or a supplied value.
         * The semantics of each are described below and the default is 'highValue'.
         * XPath and Schema
         * XPath is a W3C specification which allows the description of an element path. So to sort by title,
         * one might specify the xpath of "/record/title" within the Dublin Core schema. The records need not
         * be stored in this particular schema to be able to sort by it. The records do not even necessarily
         * need to be able to be returned in the schema.
         * SRU has the concept of utility schemas which are designed not to return records in, but into which
         * records can be transformed in order to sort them in a particular way. For example, if the record
         * has a geographical location in it, then it may be desirable to sort the locations in the records
         * from north to south and east to west. This would obviously require transformation into a schema
         * that allows sorting by a convenient coordinate system, rather than lexically on the place name,
         * and this schema may not be available for retrieving the records.
         * Missing Value Action
         * This parameter of a sort key instructs the server what to do when the supplied XPath is not present
         * within the record. For example if the server is instructed to sort by author, and a record has no author,
         * it will behave in accordance with this value.
         * Its value may be:
         * abort: The server should immediately cease trying to sort and return a diagnostic that the sort could not be performed.
         * omit: The server should remove this record from the results.
         * lowValue: The server should sort this as if it were the lowest possible value.
         * highValue: The server should sort this as if it were the highest possible value.
         * "constant": The server should sort the record as if this value ("constant") were supplied.
         * sortKeys
         * The textual representation of a sort key is achieved by the following rules.
         * The path must be included as the first parameter.
         * Subsequent parameters are separated by the use of a comma (,) character in the order given above.
         * The path and schema must be quoted if they contain quotes, commas or spaces.
         * Internal quotes must be escaped with a backslash.
         * Parameters beyond the first may be supplied with no value, in which case the server will use the default.
         * The last parameter supplied must be present. (In other words, the key may not end in a comma.)
         * Boolean parameters are expressed as 1 (true) or 0 (false).
         * Multiple keys are separated by whitespace.
         * An example of the sortKeys parameter in an SRU URL might be:
         */
        private String sortKeys;

        private String facetLimit;

        private String facetStart;

        private String facetSort;

        private String facetCount;

        private String extraRequestData;

        private String encoding;

        private String path;

        private String stylesheet;

        private String defaultStylesheet;

        private Builder() {
        }

        @SuppressWarnings("unchecked")
        public Builder setServices(Object services) {
            this.services = services instanceof Collection ? (Collection<Object>) services :
                    services != null ? Collections.singletonList(services) : Collections.emptyList();
            return this;
        }

        public Builder setRequester(SearchRequester requester) {
            this.requester = requester;
            return this;
        }

        public Builder setContent(String content) {
            this.content = content;
            return this;
        }

        public Builder setMap(Map<String, Object> map) {
            this.map = map;
            return this;
        }

        public Builder setSearchString(String searchString) {
            this.searchString = searchString;
            return this;
        }

        public Builder setSearchKey(String searchKey) {
            this.searchKey = searchKey;
            return this;
        }

        public Builder setBibliographicDescription(BibliographicDescription bibliographicDescription) {
            this.bibliographicDescription = bibliographicDescription;
            return this;
        }

        public Builder setOffset(Integer offset) {
            this.offset = offset;
            return this;
        }

        public Builder setSize(Integer size) {
            this.size = size;
            return this;
        }

        public Builder setEndpoint(String endpoint) {
            this.endpoint = endpoint;
            return this;
        }

        public Builder setFilter(String filter) {
            this.filter = filter;
            return this;
        }

        public Builder setVersion(String version) {
            this.version = version;
            return this;
        }

        public Builder setRecordPacking(String recordPacking) {
            this.recordPacking = recordPacking;
            return this;
        }

        public Builder setRecordSchema(String recordSchema) {
            this.recordSchema = recordSchema;
            return this;
        }

        public Builder setCQLQuery(String cqlQuery) {
            this.cqlQuery = cqlQuery;
            return this;
        }

        public Builder setAndFilter(Map<String, Collection<String>> andFilter) {
            this.andFilter = andFilter;
            return this;
        }

        public Builder setOrFilter(Map<String, Collection<String>> orFilter) {
            this.orFilter = orFilter;
            return this;
        }

        public Builder setStartRecord(Integer startRecord) {
            this.startRecord = startRecord;
            return this;
        }

        public Builder setMaximumRecords(Integer maximumRecords) {
            this.maximumRecords = maximumRecords;
            return this;
        }

        public Builder setSortKeys(String sortKeys) {
            this.sortKeys = sortKeys;
            return this;
        }

        public Builder setFacetLimit(String facetLimit) {
            this.facetLimit = facetLimit;
            return this;
        }

        public Builder setFacetStart(String facetStart) {
            this.facetStart = facetStart;
            return this;
        }

        public Builder setFacetSort(String facetSort) {
            this.facetSort = facetSort;
            return this;
        }

        public Builder setFacetCount(String facetCount) {
            this.facetCount = facetCount;
            return this;
        }

        public Builder setExtraRequestData(String extraRequestData) {
            this.extraRequestData = extraRequestData;
            return this;
        }

        public Builder setEncoding(String encoding) {
            this.encoding = encoding;
            return this;
        }

        public Builder setPath(String path) {
            this.path = path;
            return this;
        }

        public Builder setStyleSheet(String styleSheet) {
            this.stylesheet = styleSheet;
            return this;
        }

        public Builder setDefaultStyleSheet(String defaultStyleSheet) {
            this.defaultStylesheet = defaultStyleSheet;
            return this;
        }

        public SearchRetrieveRequest build() {
            return new SearchRetrieveRequest(this);
        }
    }
}