package org.xbib.interlibrary.action.sru;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import org.xbib.interlibrary.api.action.AbstractResponse;

public class SearchRetrieveResponse extends AbstractResponse<SearchRetrieveRequest> {

    private final SearchRetrieveResponseBuilder builder;

    SearchRetrieveResponse(SearchRetrieveRequest request, SearchRetrieveResponseBuilder builder) {
        super(request, builder.status);
        this.builder = builder;
    }

    public static SearchRetrieveResponseBuilder builder() {
        return new SearchRetrieveResponseBuilder();
    }

    @Override
    public SearchRetrieveRequest getRequest() {
        return builder.request;
    }

    public String asSruXml() throws IOException {
        StringWriter sw = new StringWriter();
        try {
            write(sw);
        } catch (XMLStreamException e) {
            throw new IOException(e);
        }
        return sw.toString();
    }

    public void write(Writer writer) throws XMLStreamException, IOException {
        if (builder.request == null) {
            return;
        }
        String version = builder.request.getVersion();
        String namespace = namespace(version);
        String schemaLocation = schemaLocation(version);
        XMLStreamWriter streamWriter = createXMLStreamWriter(writer);
        streamWriter.writeStartDocument();
        streamWriter.writeStartElement(SRU, "searchRetrieveResponse", namespace);
        streamWriter.writeAttribute("xmlns", "http://www.w3.org/XML/1998/namespace", "sru", namespace);
        streamWriter.writeAttribute("xmlns", "http://www.w3.org/XML/1998/namespace", "xsi", "http://www.w3.org/2001/XMLSchema-instance");
        streamWriter.writeAttribute("xsi", "http://www.w3.org/2001/XMLSchema-instance", "schemaLocation", schemaLocation);
        writeSRU(streamWriter, "version", version, namespace);
        writeSRU(streamWriter, "numberOfRecords", Long.toString(builder.total), namespace);
        streamWriter.writeStartElement(SRU, "records", namespace);
        for (Record record : builder.records) {
            writeRecord(writer, streamWriter, record, namespace);
        }
        if ("2.0".equals(version) && builder.facets != null && !builder.facets.isEmpty()) {
            String facetNamespace = "http://docs.oasis-open.org/ns/search-ws/facetedResults";
            streamWriter.writeStartElement(SRU, "facetedResults", namespace);
            streamWriter.writeAttribute("xmlns", "http://www.w3.org/XML/1998/namespace", "facet", facetNamespace);
            streamWriter.writeAttribute("xmlns", "http://www.w3.org/XML/1998/namespace", "xsi", "http://www.w3.org/2001/XMLSchema-instance");
            streamWriter.writeAttribute("xsi", "http://www.w3.org/2001/XMLSchema-instance", "schemaLocation", facetNamespace + " http://www.loc.gov/standards/sru/sru-2-0/schemas/facetedResults.xsd");
            streamWriter.writeStartElement(FACET, "facets", facetNamespace);
            for (Facet facet : builder.facets) {
                writeFacet(streamWriter, facet, facetNamespace);
            }
            streamWriter.writeEndElement();
            streamWriter.writeEndElement();
        }
        streamWriter.writeEndElement();
        streamWriter.writeEndElement();
        streamWriter.writeEndDocument();
        streamWriter.close();
    }

    private void writeRecord(Writer writer, XMLStreamWriter streamWriter, Record record, String recordNamespace) throws XMLStreamException, IOException {
        streamWriter.writeStartElement(SRU, "record", recordNamespace);
        writeSRU(streamWriter, "recordSchema", record.getSchema(), recordNamespace);
        writeSRU(streamWriter, "recordPacking", record.getPacking(), recordNamespace);
        streamWriter.writeStartElement(SRU, "recordData", recordNamespace);
        streamWriter.writeCharacters("");
        streamWriter.flush();
        writer.write(record.getData());
        writer.flush();
        streamWriter.writeEndElement();
        writeSRU(streamWriter, "recordIdentifier", record.getIdentifier(), recordNamespace);
        streamWriter.writeEndElement();
    }

    private void writeFacet(XMLStreamWriter streamWriter, Facet facet, String facetNamespace) throws XMLStreamException {
        streamWriter.writeStartElement(FACET, "facet", facetNamespace);
        writeFacet(streamWriter, "index", facet.getName(), facetNamespace);
        streamWriter.writeStartElement(FACET, "terms", facetNamespace);
        for (Bucket bucket : facet.getBuckets()) {
            writeFacetTerm(streamWriter, bucket, facetNamespace);
        }
        streamWriter.writeEndElement();
        streamWriter.writeEndElement();
    }

    private void writeFacetTerm(XMLStreamWriter streamWriter, Bucket bucket, String facetNamespace) throws XMLStreamException {
        streamWriter.writeStartElement(FACET, "term", facetNamespace);
        writeFacet(streamWriter, "actualTerm", bucket.getTerm(), facetNamespace);
        writeFacet(streamWriter, "requestUrl", bucket.getRequestUrl(), facetNamespace);
        writeFacet(streamWriter, "count", bucket.getCount(), facetNamespace);
        streamWriter.writeEndElement();
    }

    private void writeSRU(XMLStreamWriter streamWriter, String element, String value, String namespace) throws XMLStreamException {
        streamWriter.writeStartElement(SRU, element, namespace);
        streamWriter.writeCharacters(value != null ? value : "");
        streamWriter.writeEndElement();
    }

    private void writeFacet(XMLStreamWriter streamWriter, String element, String value, String namespace) throws XMLStreamException {
        streamWriter.writeStartElement(FACET, element, namespace);
        streamWriter.writeCharacters(value != null ? value : "");
        streamWriter.writeEndElement();
    }

    private String namespace(String version) {
        return "1.2".equals(version) ?
                "http://www.loc.gov/zing/srw/" :
                "http://docs.oasis-open.org/ns/search-ws/sruResponse";
    }

    private String schemaLocation(String version) {
        return "1.2".equals(version) ?
                "http://www.loc.gov/zing/srw/ http://www.loc.gov/standards/sru/xmlFiles/srw-types.xsd" :
                "http://docs.oasis-open.org/ns/search-ws/sruResponse http://www.loc.gov/standards/sru/sru-2-0/schemas/sruResponse.xsd";
    }

    private static XMLStreamWriter createXMLStreamWriter(Writer writer) throws XMLStreamException {
        XMLOutputFactory outputFactory = XMLOutputFactory.newFactory();
        outputFactory.setProperty("javax.xml.stream.isRepairingNamespaces", Boolean.FALSE);
        outputFactory.setProperty("com.ctc.wstx.useDoubleQuotesInXmlDecl", Boolean.TRUE);
        return outputFactory.createXMLStreamWriter(writer);
    }

    private static final String SRU = "sru";

    private static final String FACET = "facet";

}
